/*! 
  \file qpromptshow.cc
  \brief Implementation of QPromptShow class.
*/
#include <qwidget.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qmessagebox.h>
#include <fstream.h>
#include <qlistbox.h>
#include <qlabel.h>
#include <stdio.h>

#include "qfileselectwidget.h"
#include "qbigtextwidget.h"
#include "qpromptshow.h"
#include "qoptionswidget.h"

/*!
  Here the application's main window is being built. All the controls
  are placed in some layouts so that the display widget will always
  take the most of the availeble space with the control buttons in the 
  top right corner and the file name selector at the bottom.

  All the signals get connected here and if after QApplication
  argument's parsing something is left from the command line, tis
  value is put into the file selection control.
*/
QPromptShow::QPromptShow( int argc, char** argv) 
  : QApplication( argc, argv )
{
  infile = new ifstream;

  QWidget* main_widget = new QWidget;
  QGridLayout* main_layout = new QGridLayout( main_widget, 2, 2, 5 );
  main_layout->setColStretch( 0, 1 );
  main_layout->setColStretch( 1, 0 );
  main_layout->setRowStretch( 0, 1 );
  main_layout->setRowStretch( 1, 0 );
  
  QVBoxLayout* buttons_layout = new QVBoxLayout();
  main_layout->addLayout( buttons_layout, 0, 1 );

  options_button = new QPushButton( "Options", main_widget );
  options_button->setFixedSize( options_button->sizeHint() );
  options_button->setMaximumWidth( 1000 );
  connect( options_button, SIGNAL(clicked()), this, SLOT(setOptions()) );
  buttons_layout->addWidget( options_button, 0 );

  start_button = new QPushButton( "Start show", main_widget );
  start_button->setFixedSize( start_button->sizeHint() );
  start_button->setMaximumWidth( 1000 );
  connect( start_button, SIGNAL(clicked()), this, SLOT(startShow()) );
  buttons_layout->addWidget( start_button, 0 );

  end_button = new QPushButton( "Quit", main_widget );
  end_button->setFixedSize( end_button->sizeHint() );
  end_button->setMaximumWidth( 1000 );
  connect( end_button, SIGNAL(clicked()), this, SLOT(quit()) );
  buttons_layout->addWidget( end_button, 0 );

  QLabel * l = new QLabel( "Sections:", main_widget );
  l->setMinimumSize( l->sizeHint() );
  buttons_layout->addWidget( l, 0 );

  section_list = new QListBox( main_widget );
  buttons_layout->addWidget( section_list, 1 );

  //  buttons_layout->addStretch( 1 );

  file_select = new QFileSelectWidget( main_widget );
  main_layout->addWidget( file_select, 1, 0 );

  connect( file_select, SIGNAL( fileNameChanged( const char* ) ),
	   this, SLOT( rescanFile( const char* ) ) );

  if ( argc > 1 )
    file_select->setFileName( argv[1] );

  display = new QBigTextWidget( main_widget );
  main_layout->addWidget( display, 0, 0 );

  main_layout->activate();
  main_widget->show();
  setMainWidget( main_widget );
}


QPromptShow::~QPromptShow()
{
  delete infile;
}

/*!
  Creates the QOptionsWidget, fills in the values according to the
  current state of display member and executes the option dialog. If
  the execution was successfull (user ended dialog with OK button),
  the modified data is placed back in to the display widget.
*/
void QPromptShow::setOptions()
{
  QOptionsWidget options;
  options.setFontSize( display->fontSize() );
  options.setFontFamily( display->fontFamily() );
  options.setFontColor( display->fontColor() );
  options.setBgColor( display->bgColor() );
  options.setDolarColor( display->dolarColor() );
  if ( options.exec() )
    {
      display->setFontSize( options.fontSize() );
      display->setFontFamily( options.fontFamily() );
      display->setFontColor( options.fontColor() );
      display->setBgColor( options.bgColor() );
      display->setDolarColor( options.dolarColor() );
    }
}

/*!
  Starts the prompt show. Apart from opening appropriate input file
  this function also disables the controls that cannot be used during
  the show (i.e. options button and file selector). Some other
  controls are reconnected and renamed for different purposes. In
  particular <i>Start show</i> button becomes <i>Next</i> button and
  <i>Quit</i> becomes <i>Stop show</i>. The function takes into
  consideration which section was selected in section list and skips
  to that section.
*/
void QPromptShow::startShow()
{
  int count = 0;
  if ( section_list->currentItem() < 0 )
    section_list->setCurrentItem( 0 );

  infile->open( file_select->fileName(), ios::in | ios::binary | ios::nocreate  );
  if ( ! infile->is_open() )
    {
      QMessageBox::critical(0, "Error!", QString("Could not open file: ")
			    + file_select->fileName());
      return;
    }

  file_select->setEnabled( false );
  options_button->setEnabled( false );
  section_list->setEnabled( false );

  start_button->setText( "Next" );
  disconnect( start_button, SIGNAL(clicked()), this, SLOT(startShow()) );
  connect( start_button, SIGNAL(clicked()), this, SLOT(nextPrompt()) );

  end_button->setText( "Stop show" );
  disconnect( end_button, SIGNAL(clicked()), this, SLOT(quit()) );
  connect( end_button, SIGNAL(clicked()), this, SLOT(endShow()) );
 
  while ( count++ < section_list->currentItem() )
    nextSection();

  nextPrompt();
}

/*!
  This function displays the next line of the input file. If the line
  is empty it also shows the <i>End of section</i> message box. At the 
  same time the section list is being updated to reflect the section
  in which we are. At the end of the file it shows <i>End of the
  prompt file</i> message box and calls endShow() function.
*/
void QPromptShow::nextPrompt()
{
  static char buf[1024];
  int flag = 0;
  streampos last;

  do
    {
      last = infile->tellg();
      infile->getline( buf, 1023 );
      if ( ! infile->good() )
	{
	  QMessageBox::information( 0, "Info", "End of the prompt file" );
	  endShow();
	  return;
	}
      strcpy( buf, QString( buf ).simplifyWhiteSpace() );
      if ( ! buf[0] ) 
	flag = 1;
    }
  while ( ( buf[0] == '#' ) || ( ! buf[0] ) );
  
  if ( flag )
    {
      QMessageBox::information( 0, "Info", "End of this section" );

      section_list->setCurrentItem( section_list->currentItem() + 1 );
    }

  display->setText( buf );  
}

/*!
  This function closes the input file and restores the controls to the 
  default state.
  
  @see startShow()
*/
void QPromptShow::endShow()
{
  infile->close();
  //  delete infile;
  //  infile = 0;

  start_button->setText( "Start show" );
  disconnect( start_button, SIGNAL(clicked()), this, SLOT(nextPrompt()) );
  connect( start_button, SIGNAL(clicked()), this, SLOT(startShow()) );

  end_button->setText( "Quit" );
  disconnect( end_button, SIGNAL(clicked()), this, SLOT(endShow()) );
  connect( end_button, SIGNAL(clicked()), this, SLOT(quit()) );

  file_select->setEnabled( true );
  options_button->setEnabled( true );
  section_list->setEnabled( true );
}

/*!
  This function skims through specified file and looks for sections. They 
  are then given names (numbers) and put into the section_list widget.
*/
void QPromptShow::rescanFile( const char* name )
{
  static char buf[128];
  int count = 0;

  section_list->clear();

  infile->open( name, ios::in | ios::binary | ios::nocreate );
  if ( ! infile->is_open() )
    return;

  do
    {
      sprintf( buf, "Sect.: %d", ++count );
      section_list->insertItem( buf );
    }
  while ( nextSection() );
  
  infile->close();
}

/*!
  The text in the input file is divided into sections, where the
  section ends with one or more empty lines (empty means here nothing
  but a whitespaces). This function in fact just scans the file
  looking for a next empty line in it. It does not update a section
  list widget nor the display itself.
*/
int QPromptShow::nextSection()
{
  static char buf[1024];
  streampos last;

  if ( ( !infile->is_open() ) || ( !infile->good() ) )
    return 0;

  do
    {
      infile->getline( buf, 1023 );
      if ( !infile->good() )
	return 0;
      strcpy( buf, QString( buf ).simplifyWhiteSpace() );
    }
  while ( buf[0] );

  do
    {
      last = infile->tellg();
      infile->getline( buf, 1023 );
      if ( !infile->good() )
	return 0;
      strcpy( buf, QString( buf ).simplifyWhiteSpace() );
    }
  while ( !buf[0] );

  infile->seekg( last );

  return 1;
}
