/*!
  \file qbigtextwidget.cc
  \brief Implementation of QBigTextWidget class.
*/
#include <qfont.h>
#include <qpainter.h>
#include <qstrlist.h>

#include "qbigtextwidget.h"

/*!
  The constructor doesn't really do anything special. The font is set
  by default to 20pt Arial with black color on light gray
  background. The default text is set to be: Example of the <b>text</b>.
*/
QBigTextWidget::QBigTextWidget( QWidget* parent, const char* name, WFlags f )
  : QWidget( parent, name, f )
{
  font_size = 20;
  font_family = "arial";
  font_color = black;
  bg_color = lightGray;
  dolar_color = red;
  setBackgroundColor( bg_color );
  widget_text = "Example of the [text]";
}

/*!
  Does nothing.
*/
QBigTextWidget::~QBigTextWidget()
{}

/*!
  The painting of the text is where all the magic is done. In short:
  the text is split uzing whitespace and formatting delimiters and
  drawn token by token from the left edge of the widget to the
  right. If the token doesn't fit in the line it is drawn from the
  beginning of the next one. 

  BUG: The formatting delimiters are treaten as word delimiters and
  therefore a word may be split accross the lines if it contains
  formatting control sequences in the middle.

  BUG2: The colouring information is lost at each new delimiter.
*/
void QBigTextWidget::paintEvent( QPaintEvent* )
{
  QRect text_r;
  int x_offs, y_offs, min_offs, max_offs;
  const char* offset;
  int to_draw, flag;
  int to_be_underlined = 0, to_be_bold = 0, 
    to_be_italic = 0, to_be_coloured = 0;


  QFont font( font_family, font_size );
  QFontMetrics fm( font );
  QPainter p;

  p.begin( this );
  p.setFont( font );
  p.setPen( font_color );

  max_offs = 0;
  min_offs = 0;
  for (int i = 0; i<2; i++ )
    {
      x_offs = min_offs;
      y_offs = height()/2;
      offset = widget_text;
      while ( *offset )
	{
	  to_draw = 0;
	  flag = 0;
	  while ( !flag )
	    {
	      switch (offset[to_draw])
		{
		case ' ':
		case 0:
		  flag = 1;
		  break;
		case '[':
		  to_be_bold = 1;
		  flag = 1;
		  break;
		case ']':
		  to_be_bold = 0;
		  flag = 1;
		  break;
		case '{':
		  to_be_italic = 1;
		  flag = 1;
		  break;
		case '}':
		  to_be_italic = 0;
		  flag = 1;
		  break;
		case '$':
		  to_be_coloured = !to_be_coloured;
		  flag = 1;
		  break;
		case '_':
		  to_be_underlined = !to_be_underlined;
		  flag = 1;
		  break;
		default:
		  to_draw++;
		}
	    }
	  
	  fm = QFontMetrics( font );
	  text_r =  fm.boundingRect( offset, to_draw );
	  
	  if ( text_r.width()+x_offs < width() )
	    {
	      if (i) p.drawText( x_offs, y_offs, offset, to_draw );
	      x_offs += text_r.width() ;
	    }
	  else
	    {
	      if ( max_offs <= x_offs )
		max_offs = x_offs;
	      x_offs = min_offs;
	      y_offs += fm.lineSpacing();
	      if (i) p.drawText( x_offs, y_offs, offset, to_draw );
	      x_offs += text_r.width();
	    }

	  offset+=to_draw;
      
	  if (( *offset ) && ( *offset != ' ' ))
	    {
	      x_offs += fm.rightBearing( offset[-1] );
	    }

	  font.setUnderline( to_be_underlined );
	  font.setBold( to_be_bold );
	  font.setItalic( to_be_italic );
		       
	  p.setFont( font );
	  if ( to_be_coloured )
	    p.setPen( dolar_color );
	  else
	    p.setPen( font_color );
	  
	  if (( *offset ) && ( *offset != ' ' ))
	    {
	      x_offs += fm.leftBearing( offset[1] );
	      offset++;
	    }
	  

	  if ( *offset == ' ' )
	    {
	      x_offs += fm.width(' ');
	      offset++;
	    }
	}
      if ( max_offs <= x_offs )
	max_offs = x_offs;
      min_offs = (width()-max_offs)/2;
    }
  
  p.end();
}

/*!
  Sets the font size to a given value and updates the widget.
*/
void QBigTextWidget::setFontSize( int size )
{
  font_size = size;
  update();
}

/*!
  Sets the font family to a given value and updates the widget.
*/
void QBigTextWidget::setFontFamily( const char* family )
{
  font_family = family;
  update();
}

/*!
  Sets the font color to a given value and updates the widget.
*/
void QBigTextWidget::setFontColor( QColor color )
{
  font_color = color;
  update();
}

/*!
  Sets the backgroun color to a given value and updates the widget.
*/
void QBigTextWidget::setBgColor( QColor color )
{
  bg_color = color;
  setBackgroundColor( bg_color );
  update();
}

/*!
  Sets the dolar_color to a given value and updates the widget.
*/
void QBigTextWidget::setDolarColor( QColor color )
{
  dolar_color = color;
  update();
}

/*!
  Sets the displayed text to a given value and updates the widget.
*/
void QBigTextWidget::setText( const char* t )
{
  widget_text = t;
  update();
}
