/*!
  \file qbigtextwidget.h
  \brief Declaration of QBigTextWidget class.
*/
#include <qwidget.h>
#include <qstring.h>
#include <qcolor.h>

//! The class for displaying formatted text with different properties.
/*! 
  This class allows displaying any given text with a properties set
  through the following methods:
  <ul>
    <li> setFontSize - sets the size of the font in points</li>
    <li> setFontFamily - sets the family of the used font</li>
    <li> setFontColor - sets the color of the displayed text</li>
    <li> setBgColor - sets the background color of the widget</li>
    <li> setDolarColor - sets the color of the highlighted text</li>
  </ul>
  Moreover the text can be formatted using the following control
  sequences:
  <ul>
    <li><tt>[text]</tt> - the text within square brackets will be
    displayed in bold</li>
    <li><tt>{text}</tt> - the text in curly brackets will be displayed in 
    italic</li>
    <li><tt>_text_</tt> - the text between underscores will be
    displayed as underlined.</li>
    <li><tt>$text$</tt> - the text between dolar signs will be displayed in
    a color specified with setDolarColor() function</li>
  </ul>
  The control sequences work as a mode switches, so the following
  text:

  <tt>[Just] {a[n te}st] text.</tt>

  will be displayed as:

  <b>Just</b> <i>a<b>n te</i>st</b> text.
*/
class QBigTextWidget : public QWidget
{
  Q_OBJECT
public:
  //! Constructor.
  QBigTextWidget( QWidget* parent = 0, const char* name = 0, WFlags f=0 );
  //! Destructor.
  ~QBigTextWidget();

public slots:
  //! Set the font size.
  void setFontSize( int );
  //! Set the font family.
  void setFontFamily( const char* );
  //! Set the font color.
  void setFontColor( QColor );
  //! Set the background color.
  void setBgColor( QColor );
  //! Set the color of the text in '$' scope.
  void setDolarColor( QColor );
  //! Set the text to be displayed.
  void setText( const char * );


  //! Returns font_size value.
  int fontSize() const 
    { return font_size; }
  //! Returns font_family value.
  const char* fontFamily() const 
    { return font_family; }
  //! Returns font_color value.
  QColor fontColor() const 
    { return font_color; }
  //! Returns bg_color value.
  QColor bgColor() const 
    { return bg_color; }
  QColor dolarColor() const 
    { return dolar_color; }
  //! Returns widget_text value.
  const char* text() const
    { return widget_text; }

protected:
  //! Overloaded from QWidget.
  void paintEvent( QPaintEvent* );
  //! The text to be displayed.
  const char* widget_text;
  //! The size of the font in points.
  int font_size;
  //! The family of the font. @see font_families
  QString font_family;
  //! The color of the font.
  QColor font_color;
  //! The color of the background.
  QColor bg_color;
  //! The color of the text in '$' scopes.
  QColor dolar_color;
};
