/*! 
  \file qfileselectwidget.cc
  \brief Implementation of QFileSelectWidget class.
*/
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qstring.h>
#include <qfiledialog.h>

#include "qfileselectwidget.h"

/*!
  Constructs the widget and puts its children into the layout (with
  5px layout border.) It also connects all the internal signals/slots.
*/
QFileSelectWidget::QFileSelectWidget(QWidget* parent, const char* name, 
				     WFlags f ) : QWidget( parent, name, f )
{
  QHBoxLayout* layout = new QHBoxLayout( this, 5 );
  
  browse_button = new QPushButton( "Browse", this );
  browse_button->setFixedSize( browse_button->sizeHint() );
  connect( browse_button, SIGNAL(clicked()), this, SLOT(browseFileName()) );
  layout->addWidget( browse_button );
  
  file_name = new QLineEdit( this );
  connect( file_name, SIGNAL(returnPressed()),
	   this, SLOT(emitChange()) );

  layout->addWidget( file_name );
  
  layout->activate();
}

/*!
  Does nothing.
*/
QFileSelectWidget::~QFileSelectWidget()
{
}

/*!
  Sets the given text as a file name in the line edit.

  @param new_name the name to be set.
*/
void QFileSelectWidget::setFileName( const char* new_name )
{
  if ( strcmp( new_name, file_name->text() ) )
    {
      file_name->setText( new_name );
      emit fileNameChanged( new_name );
    }
}

/*!
  Returns the current value contained in the line edit widget.
*/
const char* QFileSelectWidget::fileName() const
{
  return file_name->text();
}

/*!
  Activates the file open dialog and updates the line edit widget if
  the file was selected.
*/
void QFileSelectWidget::browseFileName()
{
  QString new_name;
  new_name = QFileDialog::getOpenFileName();
  if ( ! new_name.isNull() )
    {
      file_name->setText( new_name );
      emitChange();
    }
}

/*!
  This method is used to allow for example emiting the signal as a reaction 
  to pressing <ENTER> in file_name widget.
*/
void QFileSelectWidget::emitChange()
{
#if QT_VERSION < 200
  emit fileNameChanged( file_name->text() );
#else
  emit fileNameChanged( file_name->text().ascii() );
#endif
}

/*! 
  Overloaded function dispatches enabling signal to both children so
  that enabling/disabling this widget enables/disables also the browse 
  button and the line edit.
*/
void QFileSelectWidget::setEnabled( bool e )
{
  file_name->setEnabled( e );
  browse_button->setEnabled( e );
}
