/*! 
  \file qoptionswidget.cc
  \brief Implementation of QOptionsWidget class.
*/
#include <qlayout.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qpixmap.h>

#include "qoptionswidget.h"

//! Static list of commonly used font families.
/*!
  As a matter of fact, this list should be queried from the system at
  a startup, but seemingly Qt doesn't offer this possibility (still
  searching for it), so we use here a hardcoded list of names:

  <i>"albertus",  "anagnostiko",  "antique olive",  "aplos",
  "arial",  "cg omega",  "cg times",  "charter",  "chartis",
  "chimera",  "clarendon",  "clean",  "coronet",  "courier",
  "fangsong ti",  "fixed",  "fotinos",  "fotinosbright",
  "fotinostypewriter",  "garamond",  "gothic",  "helvetica",
  "kathimerini",  "letter gothic",  "lucida",  "lucidabright",
  "lucidatypewriter",  "marigold",  "mincho",  "new century
  schoolbook",  "nil",  "open look cursor",  "open look glyph",  "song
  ti",  "symbol",  "terminal",  "times new roman",  "times",
  "univers",  "utopia",  "wingdings"</i>.
*/
const char* QOptionsWidget::font_families[] = {
  "albertus",
  "anagnostiko",
  "antique olive",
  "aplos",
  "arial",
  "cg omega",
  "cg times",
  "charter",
  "chartis",
  "chimera",
  "clarendon",
  "clean",
  "coronet",
  "courier",
  "fangsong ti",
  "fixed",
  "fotinos",
  "fotinosbright",
  "fotinostypewriter",
  "garamond",
  "gothic",
  "helvetica",
  "kathimerini",
  "letter gothic",
  "lucida",
  "lucidabright",
  "lucidatypewriter",
  "marigold",
  "mincho",
  "new century schoolbook",
  "nil",
  "open look cursor",
  "open look glyph",
  "song ti",
  "symbol",
  "terminal",
  "times new roman",
  "times",
  "univers",
  "utopia",
  "wingdings",
  0 };

#if QT_VERSION >= 200
#define QT_NAMESPACE Qt
#else
#define QT_NAMESPACE 
#endif
//! Static list of available colors.
/*! 
  This is not a graphics application, so the list of the available
  colors is limited to those that are declared in Qt as static
  objects:

  <i>black,  white,  darkGray,  gray,  lightGray,  red,  green,  blue,
  cyan,  magenta,  yellow,  darkRed,  darkGreen,  darkBlue,  darkCyan,
  darkMagenta,  darkYellow</i>. 
*/
const QColor* QOptionsWidget::colors[] = {
  &QT_NAMESPACE::black,
  &QT_NAMESPACE::white,
  &QT_NAMESPACE::darkGray,
  &QT_NAMESPACE::gray,
  &QT_NAMESPACE::lightGray,
  &QT_NAMESPACE::red,
  &QT_NAMESPACE::green,
  &QT_NAMESPACE::blue,
  &QT_NAMESPACE::cyan,
  &QT_NAMESPACE::magenta,
  &QT_NAMESPACE::yellow,
  &QT_NAMESPACE::darkRed,
  &QT_NAMESPACE::darkGreen,
  &QT_NAMESPACE::darkBlue,
  &QT_NAMESPACE::darkCyan,
  &QT_NAMESPACE::darkMagenta,
  &QT_NAMESPACE::darkYellow,
  0 };

/*!
  Constructs QOptionsWidget as a modal dialog without any parent. This 
  method further constructs and layouts the controls and initialises
  them with the default values.
*/
QOptionsWidget::QOptionsWidget() : QDialog( 0, 0, TRUE )
{
  QLabel* label;
  int i;

  QVBoxLayout* main_layout = new QVBoxLayout( this, 5 );
  QGridLayout* options_layout = new QGridLayout( 3, 4 );
  main_layout->addLayout( options_layout, 0 );
  

  label = new QLabel( "Font family", this );
  label->setMinimumSize( label->sizeHint() );
  font_family_widget = new QComboBox( TRUE, this );
  font_family_widget->insertStrList( font_families );
  font_family_widget->setFixedSize( font_family_widget->sizeHint() );
  options_layout->addWidget( label, 0, 0 );
  options_layout->addWidget( font_family_widget, 1, 0, AlignRight );

  label = new QLabel( "Font size", this );
  label->setMinimumSize( label->sizeHint() );
  font_size_widget = new QSpinBox( 10, 100, 1, this );
  font_size_widget->setFixedSize( font_size_widget->sizeHint() );
  options_layout->addWidget( label, 0, 1 );
  options_layout->addWidget( font_size_widget, 1, 1, AlignRight );

  font_color_widget = new QComboBox( this );
  bg_color_widget = new QComboBox( this );
  dolar_color_widget = new QComboBox( this );
  for ( i = 0; colors[i+1]; i++ )
    {
      QPixmap p( 30, 15 );
      p.fill( *colors[i] );
      font_color_widget->insertItem( p );
      bg_color_widget->insertItem( p );
      dolar_color_widget->insertItem( p );
    }
  font_color_widget->setFixedSize( font_color_widget->sizeHint() );
  bg_color_widget->setFixedSize( bg_color_widget->sizeHint() );
  dolar_color_widget->setFixedSize( dolar_color_widget->sizeHint() );

  label = new QLabel( "Font color", this );
  label->setMinimumSize( label->sizeHint() );
  options_layout->addWidget( label, 0, 2 );
  options_layout->addWidget( font_color_widget, 1, 2, AlignRight );
  
  label = new QLabel( "Bg color", this );
  label->setMinimumSize( label->sizeHint() );
  options_layout->addWidget( label, 0, 3 );
  options_layout->addWidget( bg_color_widget, 1, 3, AlignRight );


  label = new QLabel( "$ text $", this );
  label->setMinimumSize( label->sizeHint() );
  options_layout->addWidget( label, 2, 0 );
  options_layout->addWidget( dolar_color_widget, 2, 2 );
  
  options_layout->setColStretch( 0, 0 );
  options_layout->setColStretch( 1, 0 );
  options_layout->setColStretch( 2, 0 );
  options_layout->setColStretch( 3, 0 );
  options_layout->setRowStretch( 0, 0 );
  options_layout->setRowStretch( 1, 0 );
  options_layout->setRowStretch( 2, 0 );

  QHBoxLayout* buttons_layout = new QHBoxLayout();

  QPushButton* ok = new QPushButton( "OK", this );
  QPushButton* cancel = new QPushButton( "Cancel", this );
  connect( ok, SIGNAL(clicked()), this, SLOT(accept()) );
  connect( cancel, SIGNAL(clicked()), this, SLOT(reject()) );

  ok->setFixedSize( cancel->sizeHint() );
  cancel->setFixedSize( cancel->sizeHint() );

  main_layout->addLayout( buttons_layout, 0 );
  buttons_layout->addStretch( 1 );
  buttons_layout->addWidget( ok );
  buttons_layout->addWidget( cancel );

  main_layout->activate();
  resize( 0, 0 );
}

/*! 
  Does nothing.
*/
QOptionsWidget::~QOptionsWidget()
{
}

/*!
  This method updates bot the font_size and font_size_widget members
  according to given value.

  @param size the size of font in points.
*/
void QOptionsWidget::setFontSize( int size )
{
  font_size = size;
  font_size_widget->setValue( size );
}

/*!
  This method updates bot the font_family and font_family_widget
  members according to given value. If the family name is not in the
  static list, it is inserted at the begining of the list.

  @param family the name of the font family.
*/
void QOptionsWidget::setFontFamily( const char* family )
{
  font_family = family;
  for ( int i = 0; i< font_family_widget->count(); i++ )
    {
      if ( ! stricmp( font_family_widget->text(i), family ) )
	{
	  font_family_widget->setCurrentItem( i );
	  return;
	}
    }
  font_family_widget->insertItem( family, 0 );
}

/*!
  This method updates bot the font_color and font_color_widget members
  according to given value. If the color is not on the static list of
  colors, the default black is chosen.

  @param color the color of the font.
*/
void QOptionsWidget::setFontColor( QColor color )
{
  font_color = color;
  setSelectedColor( font_color_widget, color );
}

/*!
  This function searches through the list of static colors in order to 
  find the one that is given as a parameter. If it succeeds, the 
  appropriate combobox item is selected.
*/
void QOptionsWidget::setSelectedColor( QComboBox* widget, QColor color )
{
  for ( int i = 0; i< widget->count(); i++ )
    {
      if ( color == *colors[i] )
	{
	  widget->setCurrentItem( i );
	  return;
	}
    }
}
/*!
  This method updates bot the bg_color and bg_color_widget members
  according to given value. If the color is not on the static list of
  colors, the default black is chosen.

  @param color the color of the background.
*/
void QOptionsWidget::setBgColor( QColor color )
{
  bg_color = color;
  setSelectedColor( bg_color_widget, color );
}

/*!
  This method updates bot the dolar_color and dolar_color_widget members
  according to given value. If the color is not on the static list of
  colors, the default black is chosen.

  @param color the color of the background.
*/
void QOptionsWidget::setDolarColor( QColor color )
{
  dolar_color = color;
  setSelectedColor( dolar_color_widget, color );
}

/*!
  This function updates all members of the class according to the
  state of the controls and calls QDialog::accept().
*/
void QOptionsWidget::accept()
{
  font_size = font_size_widget->value();
  font_family = font_family_widget->currentText();
  font_color = *colors[ font_color_widget->currentItem() ];
  bg_color = *colors[ bg_color_widget->currentItem() ];
  dolar_color = *colors[ dolar_color_widget->currentItem() ];
  QDialog::accept();
}

/*!
  This function simply calls QDialog::reject(), so in a way it is
  totally not needed here, but who knows what will happen in the
  future ?
*/
void QOptionsWidget::reject()
{
  QDialog::reject();
}
