/*! \file qoptionswidget.h
  \brief Declaration of QOptionsWidget class.
*/
#include <qdialog.h>
#include <qstring.h>

class QSpinBox;
class QComboBox;

//! Widget for controlling options for the main display in the program.
/*!
  This class provides a simple layouted widgets that allow control
  over such parameters as font family, size, color and background
  color.
  All of the values controlled by the child widgets can be accessed
  via the following pairs of functions: fontSize(), setFontSize(),
  fontFamily(), setFontFamily(), fontColor(), setFontColor(),
  bgColor(), setBgColor().
 */
class QOptionsWidget : public QDialog
{
  Q_OBJECT
public:
  //! Constructor.
  QOptionsWidget();
  //! Destructor.
  ~QOptionsWidget();

  //! Returns font_size value.
  int fontSize() const
    { return font_size; }
  //! Returns font_family value.
  const char* fontFamily() const
    { return font_family; }
  //! Returns font_color value.
  QColor fontColor() const
    { return font_color; }
  //! Returns bg_color value.
  QColor bgColor() const
    { return bg_color; }
  //! Returns dolar_color value;
  QColor dolarColor() const
    { return dolar_color; }
  
public slots:
  //! Sets font_size value.
  void setFontSize( int );
  //! Sets font_family value.
  void setFontFamily( const char* );
  //! Sets font_color value.
  void setFontColor( QColor );
  //! Sets bg_color value.
  void setBgColor( QColor );
  //! Sets dolar_color value.
  void setDolarColor( QColor );

protected:
  //! Size of the font in points.
  int font_size;
  //! Family of the font (helvetica, times, etc.)
  QString font_family;
  //! Color of the font.
  QColor font_color;
  //! Color of the font in '$' scope.
  QColor dolar_color;
  //! Color of the background.
  QColor bg_color;
  //! Spinbox control for the size of the font.
  QSpinBox* font_size_widget;
  //! Combobox control for the family of the font.
  QComboBox* font_family_widget;
  //! Combobox control for the color of the font.
  QComboBox* font_color_widget;
  //! Combobox control for the color of the background.
  QComboBox* bg_color_widget;
  //! Combobox control for the color of the font in '$' scope.
  QComboBox* dolar_color_widget;
  
  //! Just a helper function.
  static void setSelectedColor( QComboBox* widget, QColor color );

  static const char* font_families[];
  static const QColor* colors[];

protected slots:
  //! Overloaded QDialog::accept() slot.
  void accept();
  //! Overloaded QDialog::reject() slot.
  void reject();
};

